@@ -14,7 +14,7 @@ class AnalyticsFunction(object):
     metadata_filename = "metadata.pkl"
     metadata_keys = ["function_name", "function_source", "function_output", "kwargs", "files", "function_elapsed_seconds"]
 
-    def __init__(self, fn, kwargs, key=None, previous_functions=None, storages=None, cachePath=None, constants=None):
+    def __init__(self, fn, kwargs, key=None, previous_functions=None, storages=None, cachePath=None, constants=None, logger=None):
         """
         Arguments:
 
@@ -23,6 +23,7 @@ class AnalyticsFunction(object):
             previous_functions - a dictionary of function keys:hashcodes for previously run functions
             cachePath - an optional Path object representing the Batch's cache path, can be blank for testing
         """
+        self.logger = logger
         self.is_populated = False
         self.key = key or fn.__name__
         self.fn = fn
@@ -39,6 +40,9 @@ class AnalyticsFunction(object):
         self.function_name = self.fn.__name__
         self.function_source = inspect.getsource(self.fn)
 
+    def __repr__(self):
+        return "<AnalyticsFunction %s> " % self.key
+
     def generate_hash(self, fn, kwargs):
         """
         Set the .h attribute containing a caching hash which will be different
@@ -51,6 +55,7 @@ class AnalyticsFunction(object):
             del kwargs['depends']
 
         self.h = hash_for_fn(fn, kwargs, self.depends_function_hashes)
+        self.logger.debug("calculated hash for %s is %s" % (self.key, self.h))
             
     def set_cache_path(self, cachePath):
         """
@@ -95,8 +100,11 @@ class AnalyticsFunction(object):
 
     def download_from_storages(self, cache_filepath):
         for storage in self.storages:
+            self.logger.debug("  from %r" % storage)
             if storage.download_cache(cache_filepath):
+                self.logger.debug("    success!")
                 return True
+        self.logger.debug("    not found")
         return False
 
     def function_metadata(self):
@@ -113,6 +121,7 @@ class AnalyticsFunction(object):
 
     def save_metadata(self):
         filepath = self.metadata_cache_filepath()
+        self.logger.debug("  saving metadata to %s" % filepath)
         with open(filepath, 'wb') as f:
             pickle.dump(self.function_metadata(), f)
         self.upload_to_storages(self.metadata_filename, filepath)
