@@ -21,7 +21,10 @@ import shutil
 import tempfile
 
 def generate_range_key(range_env):
-    return "__".join("%s_%s" % (k, range_env[k]) for k in sorted(range_env))
+    if range_env:
+        return "__".join("%s_%s" % (k, range_env[k]) for k in sorted(range_env))
+    else:
+        return ""
 
 class Batch(object):
     def __init__(self, config):
@@ -50,8 +53,17 @@ class Batch(object):
         handler.setLevel(level)
         self.logger.setLevel(level)
 
+        FORMAT = '%(asctime)-15s %(levelname)-6s %(message)s'
+        formatter = logging.Formatter(fmt=FORMAT)
+        handler.setFormatter(formatter)
+
         self.logger.addHandler(handler)
-        self.logger.info("logging!")
+        self.logger.info("")
+        self.logger.info("***** BEGIN LOGGING BATCH %s" % self.h)
+        self.logger.info("")
+        self.logger.debug("Config:")
+        for k, v in self.config.items():
+            self.logger.debug("%s: %r" % (k, v))
 
     def setup_work_dirs(self):
         self.cache_bucket_name = self.config.get('cache_bucket_name', "cache")
@@ -89,18 +101,27 @@ class Batch(object):
         for storage in self.storages:
             storage.init(self)
             storage.connect()
+            if self.config['opts'].reset:
+                self.logger.debug("resetting cache for %r" % storage)
+                storage.reset_cache()
 
     def upload_to_storages_cache(self, f):
         for storage in self.storages:
             public_url = storage.upload_cache(f.cache_filepath)
             f.public_urls.append(public_url)
 
+    def upload_to_storages_output(self, f):
+        for storage in self.storages:
+            public_url = storage.upload_output(f.canonical_filename, f.cache_filepath)
+            f.public_urls.append(public_url)
+
     def setup_document_templates(self):
-        self.logger.info("Collecting list of document templates to process...")
+        self.logger.debug("in setup_document_templates")
+        self.logger.info("collecting list of document templates to process...")
         self.template_filenames = []
 
         for key in ['templates', 'template_file', 'template_files']:
-            self.logger.info("Looking for templates specified under config key '%s'" % key)
+            self.logger.info("looking for templates specified under config key '%s'" % key)
             entries = self.config.get(key, [])
             if isinstance(entries, str):
                 entries = [entries]
@@ -113,7 +134,7 @@ class Batch(object):
             self.template_filenames += ["%s.md" % self.h]
 
         if len(self.template_filenames) == 0:
-            self.logger.info("No specified templates found, will add all in %s directory" % self.template_dir)
+            self.logger.info("no specified templates found, will add all in %s directory" % self.template_dir)
             raw_template_files = glob.glob("%s/*" % self.template_dir)
             if raw_template_files:
                 self.logger.info("  found template(s): %s" % ", ".join(raw_template_files))
@@ -127,9 +148,25 @@ class Batch(object):
 
     def run(self, analytics_modules):
         for range_env in self.range_environments():
+            self.logger.debug("")
+
+            self.logger.debug("now processing range env %s" % str(range_env))
             self.init_range(range_env)
+            self.logger.debug("range key is %s" % self.current_range_key)
+
+            self.logger.debug("")
+            self.logger.debug("***** ANALYTICS")
+            self.logger.debug("")
             self.generate_analytics(analytics_modules)
+
+            self.logger.debug("")
+            self.logger.debug("***** DOCUMENTS")
+            self.logger.debug("")
             self.generate_documents()
+
+            self.logger.debug("")
+            self.logger.debug("***** PUBLISH")
+            self.logger.debug("")
             self.publish_documents()
 
     def range_environments(self):
@@ -158,8 +195,9 @@ class Batch(object):
     def generate_analytics(self, analytics_modules):
         self.analytics_modules = analytics_modules
 
-        self.logger.debug("in generate_analytics with available modules: " + ", ".join(
-            str(m) for m in analytics_modules))
+        self.logger.debug("in generate_analytics with available modules:")
+        for m in analytics_modules:
+            self.logger.debug("     %r" % m)
 
         self.current_function_name = None
         self.current_function_data = None
@@ -180,22 +218,28 @@ class Batch(object):
     def process_analytics_entry(self, key, kwargs, previous_functions):
         af = self.resolve_function(key, kwargs, previous_functions)
 
+        self.logger.debug("checking for cached metadata from a prior run")
+        self.logger.debug("  in %s" % af.metadata_cache_filepath())
+
         if af.metadata_path_exists():
+            self.logger.debug("    success!")
             af.load_metadata()
 
         if not af.metadata_path_exists():
+            self.logger.debug("    not found")
             if af.download_from_storages(af.metadata_cache_filepath()):
                 af.load_metadata()
                 for sf in af.files.values():
                     filepath = af.supplemental_file_cache_filepath(sf.canonical_filename)
                     if not af.download_from_storages(filepath):
-                        raise Exception("Couldn't download storage for %s" % filepath)
+                        raise Exception("couldn't download storage for %s" % filepath)
 
         if not af.metadata_path_exists():
+            self.logger.debug("cached metadata not found, running the function")
             af.run_function()
             af.is_populated = True
-            af.save_metadata()
             af.from_cache = False
+            self.logger.debug("  run function complete")
         else:
             if not af.is_populated:
                 af.load_metadata()
@@ -226,13 +270,14 @@ class Batch(object):
             errmsg_raw = "couldn't find a function %s in modules %s"
             errmsg = errmsg_raw % (function_name, ", ".join(str(m) for m in self.analytics_modules))
             raise Exception(errmsg)
-        self.logger.info("matched function %s to fn %s" % (qual_function_name, str(fn)))
+        self.logger.info("matched '%s' to fn %r" % (qual_function_name, fn))
 
         return AnalyticsFunction(fn, kwargs,
             previous_functions=previous_functions, 
             storages=self.storages,
             cachePath=self.cachePath,
             constants=self.config.get('constants', None),
+            logger=self.logger,
             key=key
             )
 
@@ -278,19 +323,26 @@ class Batch(object):
         """
         for af in self.functions[self.current_range_key].values():
             for gf in af.files.values():
+                self.logger.debug("  copying supp file %s" % gf.canonical_filename)
                 shutil.copyfile(gf.cache_filepath, gf.canonical_filename)
 
-    def upload_all_supplemental_files(self):
+    def upload_all_supplemental_files(self, mode='cache'):
         """
         Uploads all supplemental files
         """
         for item in self.functions.values():
             for af in item.values():
                 for gf in af.files.values():
-                    self.upload_to_storages_cache(gf)
+                    if mode == 'cache':
+                        self.upload_to_storages_cache(gf)
+                    elif mode == 'output':
+                        self.upload_to_storages_output(gf)
+                    else:
+                        raise Exception("unexpected mode '%s'" % mode)
 
     def create_and_populate_work_dir(self, prev_doc):
         workPath = self.cachePath / "docs" / prev_doc.h
+        self.logger.debug("populating work dir %s" % workPath)
         os.makedirs(workPath, exist_ok=True)
         os.chdir(workPath)
 
@@ -362,17 +414,17 @@ class Batch(object):
             os.chdir(self.orig_dir)
 
     def rewrite_local_output(self):
-        print(os.getcwd())
         if os.path.exists(self.localOutputPath):
             if os.path.exists(self.localOutputPath / (".%s" % self.h)):
                 pass
             elif os.path.exists(self.localOutputPath / '.precipy'):
-                print("removing old %s" % self.localOutputPath)
+                self.logger.debug("  removing old %s" % self.localOutputPath)
                 shutil.rmtree(self.localOutputPath)
             else:
-                print("Can't remove old: %s" % self.localOutputPath)
+                self.logger.debug("  can't remove old %s" % self.localOutputPath)
                 return False
 
+        self.logger.debug("  copying files to %s" % (self.localOutputPath / self.current_range_key))
         shutil.copytree(self.rangeOutputPath(), self.localOutputPath / self.current_range_key)
         with open(self.localOutputPath / ".precipy", 'w') as f:
             f.write("Keep this here so precipy knows it's okay to delete this dir.")
@@ -384,27 +436,31 @@ class Batch(object):
         return True
 
     def publish_documents(self):
+        self.logger.debug("in publish_documents with range key %s" % self.current_range_key)
+
         curdir = os.getcwd()
+        self.logger.debug("changing dir to %s" % self.rangeOutputPath())
         os.chdir(self.rangeOutputPath())
 
         for doc in self.documents[self.current_range_key].values():
+            self.logger.debug("  copying file to %s" % doc.canonical_filename)
             shutil.copyfile(doc.cache_filepath, doc.canonical_filename)
         self.copy_all_supplemental_files()
 
         os.chdir(curdir)
 
-        print("output directory is %s" % self.rangeOutputPath())
+        self.logger.debug("copying output to local output directory %s" % self.localOutputPath)
         if self.rewrite_local_output():
-            print("local output directory is %s" % self.localOutputPath)
             for storage in self.storages:
-                print("processing storage %r" % storage)
+                self.logger.debug("copying output to storage %r" % storage)
                 storage.reset_output()
                 for docs in self.documents.values():
-                    print(docs)
                     for doc in docs.values():
-                        print(doc)
-                        storage.upload_output(doc.canonical_filename, doc.cache_filepath)
-                self.upload_all_supplemental_files()
+                        self.logger.debug("      processing doc %r" % doc)
+                        url = storage.upload_output(doc.canonical_filename, doc.cache_filepath)
+                        doc.public_urls.append(url)
+                        self.logger.debug("        uploaded to url %s" % url)
+                self.upload_all_supplemental_files(mode='output')
 
     def render_text(self, text):
         template = self.jinja_env.from_string(text)
